<?php
namespace Cylancer\Usertools\Utility;

use TYPO3\CMS\Core\Utility\GeneralUtility;

class RequestFEUids2Emails
{

    /**
     * Reference to the parent (calling) cObject set from TypoScript
     */
    public $cObj;

    const USERS_PATH = 'usersPath';

    const GROUPS_PATH = 'groupsPath';

    const PIDS = 'pid';

    private $queryBuilder = null;

    private $frontendUserRepository = null;

    private $frontendUserGroupRepository = null;

    /**
     * Reads the fe user uid from the get/post request.
     * You must set the USERS_PATH.
     * e.g. TYPOSCRIPT:
     *
     * lib.requestFEUids2Emails = USER
     * lib.requestFEUids2Emails {
     * pid = 17
     * usersPath = tx_powermail_pi1|field|singlereceivers
     * groupsPath = tx_powermail_pi1|field|groupreceivers
     * userFunc = Cylancer\Usertools\Utility\RequestFEUids2Emails->map
     * }
     *
     * @param
     *            string is not used.
     * @param
     *            array contains the USERS_PATH configuration.
     * @return string Returns a comma separeted list of email addresses or null
     */
    public function map($content, $conf)
    {
        if ($this->queryBuilder == null) {
            $this->queryBuilder = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Database\ConnectionPool::class)->getQueryBuilderForTable('fe_users');
        }

        if ($this->initFrontendUserRepository($conf) != false) {
            $p = $this->splitPath($conf, RequestFEUids2Emails::USERS_PATH);

            if ($p !== false) {
                $u = $this->retrieveUserEMails($this->findUids($p));
            }
            $p = $this->splitPath($conf, RequestFEUids2Emails::GROUPS_PATH);
            if ($p !== false) {
                $g = $this->retrieveGroupEMails($this->addSubgroups($this->findUids($p)));
            }
            $r = array_filter(array_unique(array_merge($u, $g)));
            return empty($r) ? null : implode(',', $r);
        } else {
            return null;
        }
    }

    private function initFrontendUserRepository(array $conf)
    {
        if (isset($conf[RequestFEUids2Emails::PIDS])) {
            $om = GeneralUtility::makeInstance(\TYPO3\CMS\Extbase\Object\ObjectManager::class);
            $this->frontendUserRepository = $om->get(\TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository::class);

            $querySettings = $om->get(\TYPO3\CMS\Extbase\Persistence\Generic\Typo3QuerySettings::class);
            $querySettings->setStoragePageIds(GeneralUtility::intExplode(',', $conf[RequestFEUids2Emails::PIDS]));
            $this->frontendUserRepository->setDefaultQuerySettings($querySettings);

            return $this->initFrontendUserGroupRepository($om, $querySettings);
        } else {
            return false;
        }
    }

    private function initFrontendUserGroupRepository($om, $querySettings)
    {
        $this->frontendUserGroupRepository = $om->get(\TYPO3\CMS\Extbase\Domain\Repository\FrontendUserGroupRepository::class);
        $this->frontendUserGroupRepository->setDefaultQuerySettings($querySettings);
        return true;
    }

    /**
     * Returns the $conf attribut with the $path as array or false if it is not posible.
     *
     * @param array $conf
     * @param String $path
     * @return boolean|boolean|array
     */
    private function splitPath(array $conf, String $path)
    {
        if (! isset($conf[$path])) {
            return false;
        }
        $r = explode('|', $conf[$path]);
        return empty($r) ? false : $r;
    }

    private function findUids(array $splittedPath)
    {
        $requestData = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP($splittedPath[0]);

        if ($requestData == null) {
            return array();
        }
        $dataCount = count($splittedPath);

        for ($i = 1; $i < $dataCount; $i ++) {
            if (is_array($requestData)) {
                $requestData = $requestData[$splittedPath[$i]];
            } else {
                return array();
            }
        }

        if (is_array($requestData)) {
            $return = array();
            foreach ($requestData as $rd) {
                $return = array_merge($return, array_filter(array_unique(array_map('trim', explode(',', $rd)))));
            }
            return $return;
        } else {
            return array_filter(array_unique(array_map('trim', explode(',', $requestData))));
        }
    }

    private function getUserEmail($uid)
    {
        if (is_numeric($uid) && intval($uid) == $uid) {
            $f = $this->frontendUserRepository;
            $u = $f->findByUid($uid);
            $u = $f->findOneByUsername($u->getUsername()); // only usable fe users
            if ($u != null) {
                $e = $u->getEmail();
                return empty($e) ? false : $e;
            }
        }
    }

    private function retrieveUserEMails(array $uids)
    {
        $r = array();
        foreach ($uids as $uid) {
            $e = $this->getUserEmail($uid);
            if ($e !== false) {
                $r[] = $e;
            }
        }
        return $r;
    }

    private function getGroupEmails(String $uid)
    {
        $r = array();
        if (is_numeric($uid) && intval($uid) == $uid) {
            $ug = $this->frontendUserGroupRepository->findByUid($uid);
            if ($ug != null) {
                $statement = $this->queryBuilder->select('uid')
                    ->from('fe_users')
                    ->where($this->queryBuilder->expr()
                    ->inSet('usergroup', $uid))
                    ->execute();
                while ($row = $statement->fetch()) {
                    $u = $this->frontendUserRepository->findByUid($row['uid']);
                    if ($u != null) {
                        $e = $u->getEmail();
                        if (! empty($e)) {
                            $r[] = $e;
                        }
                    }
                }
            }
        }
        return $r;
    }

    private function addSubgroups( Array $addGroups, Array &$groups = array())
    {
        $groups = array_unique(array_merge($groups, $addGroups));
        foreach ($addGroups as $agUid) {
            $tmp = array();
            $g = $this->frontendUserGroupRepository->findByUid($agUid);
            foreach ($this->frontendUserGroupRepository->findBySubgroup($g) as $subgroup) {
                if (! in_array($subgroup->getUid(), $groups)) {
                    $tmp[] = $subgroup->getUid();
                }
            }
            if (! empty($tmp)) {
                $this->addSubgroups($tmp, $groups);
            }
        }
        return $groups;
    }

    private function retrieveGroupEMails(array $uids)
    {
        $r = array();
        foreach ($uids as $uid) {
            $e = $this->getGroupEmails($uid);
            if (! empty($e)) {
                $r = array_merge($r, $e);
            }
        }
        return $r;
    }
}
